package page_votes

import (
	"code.electrolab.fr/it/vote.electrolab.fr/application"
	"code.electrolab.fr/it/vote.electrolab.fr/pages/common"
	"context"
	"html/template"
	"log"
	"net/http"
)

var (
	templateIndex     = template.Must(template.New("index").Parse(TEMPLATE_INDEX_STR))
	templateForbidden = template.Must(template.New("forbidden").Parse(TEMPLATE_FORBIDDEN_STR))
)

func RegisterRoutes(mux *http.ServeMux, app *application.App) {
	mux.Handle("/votes", application.NewAppHandler(app, handleGetVotes))
}

func handleGetVotes(app *application.App, ctx context.Context, w http.ResponseWriter, r *http.Request) {
	var (
		err               error
		templateIndexVars = new(TemplateIndexVars)
	)

	if voterPublicId, voterFullName, err := common.GetVoterInfo(app, ctx, r); err != nil {
		// Token is not valid
		w.WriteHeader(http.StatusForbidden)
		templateForbidden.Execute(w, nil)
		return
	} else {
		templateIndexVars.VoterPublicId = voterPublicId
		templateIndexVars.VoterPrivateToken = r.FormValue("private_token")
		templateIndexVars.VoterFullname = voterFullName
	}

	// Token is valid, download the questions

	rows, err := app.DB.QueryContext(ctx, `SELECT name, position, title, sentence, vote_open, vote_count, COALESCE((SELECT TRUE FROM votes WHERE votes.question_name=questions.name AND voter_token=$1), FALSE) AS voted_already FROM questions ORDER BY position`, templateIndexVars.VoterPublicId)
	if err != nil {
		// Query has an error
		log.Printf("Query error 2: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}
	defer rows.Close()

	for rows.Next() {
		var question TemplateIndexVarsQuestion

		err = rows.Scan(&question.Name, &question.Position, &question.Title, &question.Sentence, &question.VoteOpen, &question.VoteCount, &question.VotedAlready)
		if err != nil {
			log.Printf("Query error 3: %s", err.Error())
			w.WriteHeader(http.StatusInternalServerError)
			return
		}

		templateIndexVars.Questions = append(templateIndexVars.Questions, question)
	}

	err = templateIndex.Execute(w, templateIndexVars)
	if err != nil {
		// Template has an error
		log.Printf("Template execution error: %s", err.Error())
		w.WriteHeader(http.StatusInternalServerError)
		return
	}

}
